import datetime
import time
from data_mod import DataMod
import sys
import os
import glob
import sets
from dateutil.relativedelta import *
from cdo import *
import copy
from Scientific.N import *
from Scientific.IO.NetCDF import *
import numpy
import shutil
from common import *
#from nco import Nco


class DataEC3(DataMod):
     """ classe of data for all experiments realized 
     with autosubmit with the standard outclass 
     """
     def __init__(self, dataname, varname):
         DataMod.__init__(self, dataname, varname)
         self.set_freq(datetime.timedelta(0, 0, 0, 0, 0, 6))

     def set_inputdirAuto(self):
         """defining the path to look for the model daily data"""
         dataname=self.get_dataname()
         varname=self.get_varname()
         
         inputdir = "/cfunas/exp/ecearth/gribfiles/"+dataname+"/"

         if (not os.path.isdir(inputdir)):
             print "This directory does not exist:"
             print inputdir
             sys.exit(0)

         self.set_inputdir(inputdir)

     def set_outputdirAuto(self):
         """defining the path to store the output"""
         dataname=self.get_dataname()
         varname=self.get_varname()
         
         outputdir = "/cfunas/exp/ecearth/"+dataname+"/"+""

         if (not os.path.isdir(inputdir)):
             print "This directory does not exist:"
             print inputdir
             sys.exit(0)

         self.set_outputdir(outputdir)

     def set_files(self, date1, date2):
        """set the list of stardates and correponding directories
        as for these type of experiments the members and leadtime
        are stored in different file we first find the correct 
        path to the directories"""

        inputdir=self.get_inputdir()
        filelist=[]
        datelist=[]
        filetype="dir"


        if (not os.path.isdir(inputdir)):
             print "This directory does not exist:"
             print inputdir
             sys.exit(0)

        filebase=inputdir+str(date1).replace("-","")+"/fc0/outputs/"
        filelist.append(filebase)
        datelist.append(date1)
        d=date1 + relativedelta(years=+1)
        #loop on the date to find all the files
        while d <= date2:
             filebase=inputdir+str(d).replace("-","")+"/fc0/outputs/"
             datelist.append(d)
             filelist.append(filebase)
             d = d + relativedelta(years=+1)

        self.set_filetype(filetype)
        self.set_filelist(filelist)
        self.set_sdates(datelist)
     
     def format_files_nc(self):
         """transform the grb file 
         into ncdf file readable for cdo"""
         
         cdo=Cdo()

         #get the needed variables from the object
         inputdir=self.get_inputdir()
         dirlist=self.get_filelist()
         sdates=self.get_sdates()
         dataname=self.get_dataname()
         varname=self.get_varname()
         nmember=self.get_nmember()
         filetype=self.get_filetype()
         filelist=[]
         #split the variable into varaux and level if needed
         #if ("0" in var) | ("5" in var):
              

         #define name of var in the file
         dictvar={"tas":"T2M","lsp":"LSP","cp":"CP","psl":"MSL",
                  "ta":"T", "ua":"U", "va":"V"}
         oldname=dictvar.get(varname)
         if oldname == None:
              print("the variable "+varname+"is no available in the format_file function")
              sys.exit(0)

         #the files are stored in on directory
         #this function works only for these configuration
         #sdates/menbers/file per leadtime
         if (filetype != "dir"):
              print("the function is done for directory type of file")
              print("not "+filetype+"type of file")
              sys.exit(0)

         freq=self.get_freq()
         #get the number of members the function
         #all sdates must have the same number of members
         nmember=len(glob.glob(inputdir+str(sdates[0]).replace("-","")+"/*"))
         print(nmember)
         
         for idir in range(len(dirlist)):
             direcfc0=dirlist[idir]
             date=sdates[idir]
             print("idir")
             print(idir)
             lstmemb=[]
             for imemb in range(nmember):
                  print("imemb")
                  print(imemb)

                  membname="fc%i"%imemb
                  direc=direcfc0.replace("fc0",membname)
                  
                  #select if SH or GG files
                  if ("ta" in varname) | ("ua" in varname) | ("va" in varname) | ("ga" in varname):
                       vartype="SH"
                  else:
                       vartype="GG"
                  
                  #count the nb of file to know the leadtime
                  filelistmon=glob.glob(direc+'*'+vartype+"*")
                  filelistmon.sort()
                  print(filelistmon)
                  #filelistmon=filelistmon.sort()
                  nbmon=2 #len(filelistmon)
                  
                  filelistmonout=[]
                  #concatenate the leadtimes
                  for ifile in range(nbmon):
                       print("ifile")
                       print(ifile)
                       fileout=filelistmon[ifile].split("/")[-1]
                       fileout=fileout.replace(".grb",".nc")
                       #include sellevl here, when needed
                       fileout=cdo.selvar(oldname,
                                    input=filelistmon[ifile], 
                                    options="-R -r -t ecmwf -f nc")
                       
                       filelistmonout.append(fileout)


                  #to be continued tommorow!!!!! bug in time axis!!!!

                  filemembout=cdo.mergetime(input=filelistmonout, 
                                     options= "-O -r -f nc")
                  print(filemembout)
                  print(filelistmonout)
                  #sys.exit(0)
                  
                  lstmemb.append(filemembout)

             fileout=varname + "_" + str(date).replace("-","")+".nc"
             cdo.chvar(oldname,varname, 
                       input=cdo.merge(input=lstmemb,
                                       options="-O -r"),
                       output=fileout, options="-r", 
                       force =  False)
             
             filelist.append(fileout)
         print(filelist)
         self.set_filelist(filelist)
         self.set_freq(get_freq(filelist[0]))
         self.set_nmember(nmember)
         self.see()

         
     def ready(self, date1, date2):
         """ make the Data object ready to calculate statistic
         """
         cdo=Cdo()

         dataname=self.get_dataname()
         varname=self.get_varname()

         if varname == "tasmin":
              self.set_varname("tas")
              self.set_inputdirAuto()
              self.set_outputdirAuto() #look for the output directory
              self.set_files(date1,date2) #look for the files between date1 and date2 
              self.format_files_nc() #transform grib in nc files cdo compatible
              self.cdo_statistic("daymin","tasmin")
         elif varname == "tasmax":
              self.set_varname("tas")
              self.set_inputdirAuto()
              self.set_outputdirAuto() #look for the output directory
              self.set_files(date1,date2) #look for the files between date1 and date2 
              self.format_files_nc() #transform grib in nc files cdo compatible
              self.cdo_statistic("daymax","tasmax")
         elif varname == "prlr":
              auxlsp=DataEC3(dataname,"lsp")
              auxlsp.set_inputdirAuto()
              auxlsp.set_files(date1,date2) #look for the files between date1 and date2 
              auxlsp.format_files_nc() #transform grib in nc files cdo compatible

              auxcp=DataEC3(dataname,"cp")
              auxcp.set_inputdirAuto()
              auxcp.set_files(date1,date2) #look for the files between date1 and date2 
              auxcp.format_files_nc() #transform grib in nc files cdo compatible

              aux=weighted_sum((auxlsp, auxcp), (1,1), varname)
              self.copy(aux)
              self.cdo_statistic("daysum","prlr")
              self.set_outputdir("/cfunas/exp/ecearth/"+dataname+"/monthly_statistics"+"") #look for the output directory
         else:
              self.set_varname(varname)
              self.set_inputdirAuto()
              self.set_outputdirAuto() #look for the output directory
              self.set_files(date1,date2) #look for the files between date1 and date2 
              self.format_files_nc() #transform grib in nc files cdo compatible
         self.see()
